#include "serial.h"
#include <iostream>

Serial::Serial()
    : m_hEvThread(0)
{
    m_params.hPort = INVALID_HANDLE_VALUE;
    m_params.setMask = EV_ERR | EV_RXCHAR | EV_TXEMPTY | EV_BREAK | EV_CTS | EV_DSR | EV_RING | EV_RLSD | EV_RXFLAG;
}

bool Serial::open(const char *name)
{
    m_params.hPort = ::CreateFileA(name, GENERIC_READ | GENERIC_WRITE,
                                   0, 0, OPEN_EXISTING, 0 /* FILE_FLAG_OVERLAPPED */, 0);

    if (INVALID_HANDLE_VALUE == m_params.hPort)
        return false;

    DCB dcb;

    if (::GetCommState(m_params.hPort, &dcb) == 0) {
        close();
        return false;
    }

    dcb.BaudRate = CBR_9600;
    dcb.ByteSize = 8;
    dcb.DCBlength = sizeof(DCB);
    dcb.fAbortOnError = false;
    dcb.fBinary = true;
    dcb.fDsrSensitivity = false;
    dcb.fDtrControl = false;
    dcb.fOutxCtsFlow = false;
    dcb.fOutxDsrFlow = false;
    dcb.fParity = false;
    dcb.fRtsControl = false;
    dcb.Parity = NOPARITY;
    dcb.StopBits = ONESTOPBIT;

    if (::SetCommState(m_params.hPort, &dcb) == 0) {
        close();
        return false;
    }

    if (::SetCommMask(m_params.hPort, m_params.setMask) == 0) {
        close();
        return false;
    }

    m_hEvThread = ::CreateThread(0, 0, (LPTHREAD_START_ROUTINE)&eventThreadHandler, &m_params, 0, 0);
    if (!m_hEvThread) {
        close();
        return false;
    }
    return true;
}

void Serial::close()
{
    if (m_hEvThread) {
        ::TerminateThread(m_hEvThread, -1);
        m_hEvThread = 0;
    }

    if (INVALID_HANDLE_VALUE != m_params.hPort) {
        ::CancelIo(m_params.hPort);
        ::CloseHandle(m_params.hPort);
    }
}

bool Serial::setRate(int rate)
{
    // Нужно проверить, а прерывает ли SetCommMask() в WinCE
    // функцию WaitCommEvent() из обработчика eventThreadHandler() ?
    // Потому что в обычном Win32, в синхронном режиме, SetCommMask()
    // ничего не прерывает и блокирует навечно основную нить приложения.
    //
    ::SetCommMask(m_params.hPort, 0);
    
    std::cout << "SetCommMask worked. " << std::endl;

    DCB dcb;
    if (::GetCommState(m_params.hPort, &dcb)) {
        dcb.BaudRate = rate;
        return (::SetCommState(m_params.hPort, &dcb) > 0);
    }
    return false;
}

DWORD Serial::eventThreadHandler(LPVOID params)
{
    DWORD currMask = 0;
    PrivateData *ptr = (PrivateData *)params;
    while (::WaitCommEvent(ptr->hPort, &currMask, 0)) {
    
        std::cout << "WaitCommEvent worked ok." << std::endl;
        
        ::SetCommMask(ptr->hPort, ptr->setMask);

        if (EV_ERR & currMask) {
            DWORD err;
            COMSTAT cs;
            ::ClearCommError(ptr->hPort, &err, &cs);
            std::cout << "EV_ERR" << std::endl;
        }

        if (EV_RXCHAR & currMask) {
            DWORD rb;
            char c;
            if (::ReadFile(ptr->hPort, &c, 1, &rb, 0))
                std::cout << "EV_RXCHAR: " << (int)c << " len " << rb << std::endl;
        }

        if (EV_TXEMPTY & currMask)
            std::cout << "EV_TXEMPTY" << std::endl;

    }
    std::cout << "WaitCommEvent worked error." << std::endl;
    return -1;
}
